@@ -9,6 +9,7 @@ import subprocess
 import pathlib
 import traceback  # noqa
 import locale
+import re
 import sys
 import gi
 
@@ -134,13 +135,28 @@ class KlipperScreen(Gtk.Window):
         logging.info(f"Screen resolution: {self.width}x{self.height}")
         self.theme = self._config.get_main_config().get('theme')
         self.show_cursor = self._config.get_main_config().getboolean("show_cursor", fallback=False)
+        self.setup_gtk_settings()
+        self.style_provider = Gtk.CssProvider()
         self.gtk = KlippyGtk(self)
-        self.init_style()
+        self.base_css = ""
+        self.load_base_styles()
         self.set_icon_from_file(os.path.join(klipperscreendir, "styles", "icon.svg"))
-
-        self.base_panel = BasePanel(self, title="Base Panel")
+        self.base_panel = BasePanel(self)
+        self.change_theme(self.theme)
         self.add(self.base_panel.main_grid)
         self.show_all()
+        min_ver = (3, 8)
+        if sys.version_info < min_ver:
+            self.show_error_modal(
+                "Error",
+                _("The system doesn't meet the minimum requirement") + "\n"
+                + _("Minimum:") + f" Python {min_ver[0]}.{min_ver[1]}" + "\n"
+                + _("System:") + f" Python {sys.version_info.major}.{sys.version_info.minor}"
+            )
+            return
+        if self._config.errors:
+            self.show_error_modal("Invalid config file", self._config.get_errors())
+            return
         if self.show_cursor:
             self.get_window().set_cursor(
                 Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.ARROW))
@@ -150,18 +166,9 @@ class KlipperScreen(Gtk.Window):
                 Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.BLANK_CURSOR))
             os.system("xsetroot  -cursor ks_includes/emptyCursor.xbm ks_includes/emptyCursor.xbm")
         self.base_panel.activate()
-        if self._config.errors:
-            self.show_error_modal("Invalid config file", self._config.get_errors())
-            # Prevent this dialog from being destroyed
-            self.dialogs = []
         self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
         self.log_notification("KlipperScreen Started", 1)
         self.initial_connection()
-        if sys.version_info == (3, 7):
-            GLib.timeout_add_seconds(2, self.show_popup_message,
-                                     _("Warning") + f" Python 3.7\n"
-                                     + _("Ended official support in June 2023") + "\n"
-                                     + _("KlipperScreen will drop support in June 2024"), 2)
 
     def state_execute(self, state, callback):
         self.close_screensaver()
@@ -298,13 +305,18 @@ class KlipperScreen(Gtk.Window):
             raise FileNotFoundError(os.strerror(2), "\n" + panel_path)
         return import_module(f"panels.{panel}")
 
-    def show_panel(self, panel, title, remove_all=False, panel_name=None, **kwargs):
+    def show_panel(self, panel, title=None, remove_all=False, panel_name=None, **kwargs):
         if panel_name is None:
             panel_name = panel
         try:
             if remove_all:
-                self._remove_all_panels()
                 self.panels_reinit = list(self.panels)
+                if panel in self._cur_panels:
+                    self._menu_go_back(home=True)
+                else:
+                    self._remove_all_panels()
+                    for dialog in self.dialogs:
+                        self.gtk.remove_dialog(dialog)
             else:
                 self._remove_current_panel()
             if panel_name not in self.panels:
@@ -314,7 +326,7 @@ class KlipperScreen(Gtk.Window):
                     self.show_error_modal(f"Unable to load panel {panel}", f"{e}\n\n{traceback.format_exc()}")
                     return
             elif panel_name in self.panels_reinit:
-                logging.info("Reinitializing panel")
+                logging.info(f"Reinitializing panel {panel}")
                 self.panels[panel_name].__init__(self, title, **kwargs)
                 self.panels_reinit.remove(panel_name)
             self._cur_panels.append(panel_name)
@@ -322,7 +334,14 @@ class KlipperScreen(Gtk.Window):
         except Exception as e:
             logging.exception(f"Error attaching panel:\n{e}\n\n{traceback.format_exc()}")
 
+    def set_panel_title(self, title):
+        self.base_panel.set_title(title)
+
     def attach_panel(self, panel):
+        if panel in self.panels_reinit:
+            # this happens when the first panel needs a reinit
+            self.reload_panels()
+            return
         self.base_panel.add_content(self.panels[panel])
         logging.debug(f"Current panel hierarchy: {' > '.join(self._cur_panels)}")
         while len(self.panels[panel].menu) > 1:
@@ -341,10 +360,12 @@ class KlipperScreen(Gtk.Window):
         self.notification_log.append(log_entry)
         self.process_update("notify_log", log_entry)
 
-    def show_popup_message(self, message, level=3):
-        if (datetime.now() - self.last_popup_time).seconds < 1:
-            return
-        self.last_popup_time = datetime.now()
+    def show_popup_message(self, message, level=3, from_ws=False):
+        if from_ws:
+            if (datetime.now() - self.last_popup_time).seconds < 1:
+                return
+            self.last_popup_time = datetime.now()
+
         self.close_screensaver()
         if self.popup_message is not None:
             self.close_popup_message()
@@ -389,22 +410,24 @@ class KlipperScreen(Gtk.Window):
 
     def close_popup_message(self, widget=None):
         if self.popup_message is None:
-            return
+            return False
         self.popup_message.popdown()
         if self.popup_timeout is not None:
             GLib.source_remove(self.popup_timeout)
-        self.popup_message = self.popup_timeout = None
+            self.popup_timeout = None
+        self.popup_message = None
         return False
 
-    def show_error_modal(self, err, e=""):
-        logging.error(f"Showing error modal: {err} {e}")
+    def show_error_modal(self, title_msg, description="", help_msg=None):
+        logging.error(f"Showing error modal: {title_msg} {description}")
 
         title = Gtk.Label(wrap=True, wrap_mode=Pango.WrapMode.CHAR, hexpand=True, halign=Gtk.Align.START)
-        title.set_markup(f"<b>{err}</b>\n")
+        title.set_markup(f"<b>{title_msg}</b>\n")
         version = Gtk.Label(label=f"{functions.get_software_version()}", halign=Gtk.Align.END)
 
-        help_msg = _("Provide KlipperScreen.log when asking for help.\n")
-        message = Gtk.Label(label=f"{help_msg}\n\n{e}", wrap=True, wrap_mode=Pango.WrapMode.CHAR)
+        if not help_msg:
+            help_msg = _("Provide KlipperScreen.log when asking for help.\n")
+        message = Gtk.Label(label=f"{description}\n\n{help_msg}", wrap=True, wrap_mode=Pango.WrapMode.CHAR)
         scroll = self.gtk.ScrolledWindow()
         scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         scroll.add(message)
@@ -416,13 +439,13 @@ class KlipperScreen(Gtk.Window):
         grid.attach(scroll, 0, 2, 2, 1)
 
         buttons = [
-            {"name": _("Go Back"), "response": Gtk.ResponseType.CANCEL}
+            {"name": _("Close"), "response": Gtk.ResponseType.CLOSE}
         ]
         self.gtk.Dialog(_("Error"), buttons, grid, self.error_modal_response)
 
-    def error_modal_response(self, dialog, response_id):
-        self.gtk.remove_dialog(dialog)
-        self.restart_ks()
+    @staticmethod
+    def error_modal_response(dialog, response_id):
+        os._exit(1)
 
     def restart_ks(self, *args):
         logging.debug(f"Restarting {sys.executable} {' '.join(sys.argv)}")
@@ -430,64 +453,92 @@ class KlipperScreen(Gtk.Window):
         # noinspection PyUnreachableCode
         self._ws.send_method("machine.services.restart", {"service": "KlipperScreen"})  # Fallback
 
-    def init_style(self):
+    def setup_gtk_settings(self):
         settings = Gtk.Settings.get_default()
         settings.set_property("gtk-theme-name", "Adwaita")
         settings.set_property("gtk-application-prefer-dark-theme", False)
-        css_data = pathlib.Path(os.path.join(klipperscreendir, "styles", "base.css")).read_text()
 
-        with open(os.path.join(klipperscreendir, "styles", "base.conf")) as f:
-            style_options = json.load(f)
-        # Load custom theme
-        theme = os.path.join(klipperscreendir, "styles", self.theme)
-        theme_style = os.path.join(theme, "style.css")
-        theme_style_conf = os.path.join(theme, "style.conf")
-
-        if os.path.exists(theme_style):
-            with open(theme_style) as css:
-                css_data += css.read()
-        if os.path.exists(theme_style_conf):
-            try:
-                with open(theme_style_conf) as f:
-                    style_options.update(json.load(f))
-            except Exception as e:
-                logging.error(f"Unable to parse custom template conf file:\n{e}\n\n{traceback.format_exc()}")
-
-        self.gtk.color_list = style_options['graph_colors']
-
-        for i in range(len(style_options['graph_colors']['extruder']['colors'])):
-            num = "" if i == 0 else i
-            css_data += "\n.graph_label_extruder%s {border-left-color: #%s}" % (
-                num,
-                style_options['graph_colors']['extruder']['colors'][i]
-            )
-        for i in range(len(style_options['graph_colors']['bed']['colors'])):
-            css_data += "\n.graph_label_heater_bed%s {border-left-color: #%s}" % (
-                "" if i == 0 else i + 1,
-                style_options['graph_colors']['bed']['colors'][i]
-            )
-        for i in range(len(style_options['graph_colors']['fan']['colors'])):
-            css_data += "\n.graph_label_fan_%s {border-left-color: #%s}" % (
-                i + 1,
-                style_options['graph_colors']['fan']['colors'][i]
-            )
-        for i in range(len(style_options['graph_colors']['sensor']['colors'])):
-            css_data += "\n.graph_label_sensor_%s {border-left-color: #%s}" % (
-                i + 1,
-                style_options['graph_colors']['sensor']['colors'][i]
-            )
-
-        css_data = css_data.replace("KS_FONT_SIZE", f"{self.gtk.font_size}")
-
-        style_provider = Gtk.CssProvider()
-        style_provider.load_from_data(css_data.encode())
+    def load_base_styles(self):
+        base_conf_path = os.path.join(klipperscreendir, "styles", "base.conf")
+        with open(base_conf_path) as f:
+            self.style_options = json.load(f)
+        self.gtk.color_list = self.style_options['graph_colors']
+        base_css_path = os.path.join(klipperscreendir, "styles", "base.css")
+        self.base_css = pathlib.Path(base_css_path).read_text()
+        self.base_css = self.base_css.replace("KS_FONT_SIZE", f"{self.gtk.font_size}")
+        self.base_css = self.customize_graph_colors(self.base_css)
 
+        self.style_provider.load_from_data(self.base_css.encode())
         Gtk.StyleContext.add_provider_for_screen(
             Gdk.Screen.get_default(),
-            style_provider,
+            Gtk.CssProvider(),
             Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
         )
 
+    def load_custom_theme(self, theme_name):
+        theme_dir = os.path.join(klipperscreendir, "styles", theme_name)
+        theme_css_path = os.path.join(theme_dir, "style.css")
+        theme_conf_path = os.path.join(theme_dir, "style.conf")
+
+        theme_css = ""
+        theme_options = {}
+
+        if os.path.exists(theme_css_path):
+            theme_css = pathlib.Path(theme_css_path).read_text()
+
+        if os.path.exists(theme_conf_path):
+            try:
+                with open(theme_conf_path) as f:
+                    theme_options = json.load(f)
+            except Exception as e:
+                logging.error(
+                    f"Unable to parse custom template conf file:\n"
+                    f"{e}\n\n"
+                    f"{traceback.format_exc()}"
+                )
+
+        return theme_css, theme_options
+
+    def customize_graph_colors(self, css_data):
+        for category, category_data in self.style_options['graph_colors'].items():
+            for i, color in enumerate(category_data['colors'], start=0):
+                if category == "extruder":
+                    class_name = f".graph_label_{category}{i}" if i > 0 else f".graph_label_{category}"
+                elif category == "bed":
+                    class_name = f".graph_label_heater_{category}"
+                else:
+                    class_name = f".graph_label_{category}_{i + 1}"
+                css_data += f"\n{class_name} {{ border-left-color: #{color} }}"
+        return css_data
+
+    def update_style_provider(self, theme_css):
+        css_data = self.customize_graph_colors(theme_css)
+        css_data = self.base_css + css_data
+        screen = Gdk.Screen.get_default()
+        if self.style_provider:
+            Gtk.StyleContext.remove_provider_for_screen(screen, self.style_provider)
+        self.style_provider = Gtk.CssProvider()
+        self.style_provider.load_from_data(css_data.encode())
+        Gtk.StyleContext.add_provider_for_screen(
+            screen,
+            self.style_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
+
+    def change_theme(self, theme_name=None):
+        if not theme_name:
+            theme_name = self._config.get_main_config().get('theme')
+        self.gtk.update_themedir(theme_name)
+        theme_css, theme_options = self.load_custom_theme(theme_name)
+        self.style_options.update(theme_options)
+        self.gtk.color_list = self.style_options['graph_colors']
+        self.update_style_provider(theme_css)
+        self.reload_icon_theme()
+
+    def reload_icon_theme(self):
+        self.panels_reinit = list(self.panels)
+        self.base_panel.reload_icons()
+
     def _go_to_submenu(self, widget, name):
         logging.info(f"#### Go to submenu {name}")
         # Find current menu item
@@ -507,20 +558,20 @@ class KlipperScreen(Gtk.Window):
             logging.info("No items in menu")
 
     def _remove_all_panels(self):
-        for _ in self.base_panel.content.get_children():
-            self.base_panel.content.remove(_)
-        for dialog in self.dialogs:
-            self.gtk.remove_dialog(dialog)
-        for panel in list(self.panels):
-            if hasattr(self.panels[panel], "deactivate"):
-                self.panels[panel].deactivate()
+        logging.debug("Removing all panels")
+        while len(self._cur_panels) > 0:
+            self._remove_current_panel()
+            del self._cur_panels[-1]
         self._cur_panels.clear()
         self.close_screensaver()
+        gc.collect()
 
     def _remove_current_panel(self):
-        self.base_panel.remove(self.panels[self._cur_panels[-1]].content)
+        if not self._cur_panels:
+            return
         if hasattr(self.panels[self._cur_panels[-1]], "deactivate"):
             self.panels[self._cur_panels[-1]].deactivate()
+        self.base_panel.remove(self.panels[self._cur_panels[-1]].content)
 
     def _menu_go_back(self, widget=None, home=False):
         logging.info(f"#### Menu go {'home' if home else 'back'}")
@@ -530,22 +581,28 @@ class KlipperScreen(Gtk.Window):
             del self._cur_panels[-1]
             if not home:
                 break
-        if len(self._cur_panels) < 1:
-            self.reload_panels()
-            return
         self.attach_panel(self._cur_panels[-1])
 
     def reset_screensaver_timeout(self, *args):
         if self.screensaver_timeout is not None:
             GLib.source_remove(self.screensaver_timeout)
             self.screensaver_timeout = None
-        if not self.use_dpms and self._config.get_main_config().get('screen_blanking') != "off":
+        if self.use_dpms:
+            return
+        if self.printer and self.printer.state in ("printing", "paused"):
+            use_screensaver = self._config.get_main_config().get('screen_blanking_printing') != "off"
+        else:
+            use_screensaver = self._config.get_main_config().get('screen_blanking') != "off"
+        if use_screensaver:
             self.screensaver_timeout = GLib.timeout_add_seconds(self.blanking_time, self.show_screensaver)
 
     def show_screensaver(self):
         logging.debug("Showing Screensaver")
         if self.screensaver is not None:
             self.close_screensaver()
+        if self.screensaver_timeout is not None:
+            GLib.source_remove(self.screensaver_timeout)
+            self.screensaver_timeout = None
         if self.blanking_time == 0:
             return False
         self.remove_keyboard()
@@ -656,7 +713,7 @@ class KlipperScreen(Gtk.Window):
 
     def show_printer_select(self, widget=None):
         self.base_panel.show_heaters(False)
-        self.show_panel("printer_select", _("Printer Select"), remove_all=True)
+        self.show_panel("printer_select", remove_all=True)
 
     def websocket_connection_cancel(self):
         self.printer_initializing(
@@ -705,12 +762,10 @@ class KlipperScreen(Gtk.Window):
     def state_paused(self):
         self.state_printing()
         if self._config.get_main_config().getboolean("auto_open_extrude", fallback=True):
-            self.show_panel("extrude", _("Extrude"))
+            self.show_panel("extrude")
 
     def state_printing(self):
-        for dialog in self.dialogs:
-            self.gtk.remove_dialog(dialog)
-        self.show_panel("job_status", _("Printing"), remove_all=True)
+        self.show_panel("job_status", remove_all=True)
 
     def state_ready(self, wait=True):
         # Do not return to main menu if completing a job, timeouts/user input will return
@@ -721,7 +776,7 @@ class KlipperScreen(Gtk.Window):
             self.printer.state = "not ready"
             return
         self.files.refresh_files()
-        self.show_panel("main_menu", None, remove_all=True, items=self._config.get_menu_items("__main"))
+        self.show_panel("main_menu", remove_all=True, items=self._config.get_menu_items("__main"))
 
     def state_startup(self):
         self.printer_initializing(_("Klipper is attempting to start"))
@@ -729,7 +784,6 @@ class KlipperScreen(Gtk.Window):
     def state_shutdown(self):
         self.printer.stop_tempstore_updates()
         msg = self.printer.get_stat("webhooks", "state_message")
-        msg = msg if "ready" not in msg else ""
         self.printer_initializing(_("Klipper has shutdown") + "\n\n" + msg, go_to_splash=True)
 
     def toggle_shortcut(self, show):
@@ -755,9 +809,13 @@ class KlipperScreen(Gtk.Window):
         if "printer_select" in self._cur_panels:
             self.show_printer_select()
             return
+        home = self._cur_panels[0]
+        self.panels_reinit = list(self.panels)
         self._remove_all_panels()
-        if self.printer is not None:
-            self.printer.change_state(self.printer.state)
+        if home == "main_menu":
+            self.show_panel(home, items=self._config.get_menu_items("__main"))
+        else:
+            self.show_panel(home)
 
     def _websocket_callback(self, action, data):
         if self.connecting:
@@ -779,9 +837,9 @@ class KlipperScreen(Gtk.Window):
         elif action == "notify_status_update" and self.printer.state != "shutdown":
             self.printer.process_update(data)
             if 'manual_probe' in data and data['manual_probe']['is_active'] and 'zcalibrate' not in self._cur_panels:
-                self.show_panel("zcalibrate", _('Z Calibrate'))
+                self.show_panel("zcalibrate")
             if "screws_tilt_adjust" in data and 'bed_level' not in self._cur_panels:
-                self.show_panel("bed_level", _('Bed Level'))
+                self.show_panel("bed_level")
         elif action == "notify_filelist_changed":
             if self.files is not None:
                 self.files.process_update(data)
@@ -792,7 +850,7 @@ class KlipperScreen(Gtk.Window):
         elif action == "notify_update_response":
             if 'message' in data and 'Error' in data['message']:
                 logging.error(f"{action}:{data['message']}")
-                self.show_popup_message(data['message'], 3)
+                self.show_popup_message(data['message'], 3, from_ws=True)
                 if "KlipperScreen" in data['message']:
                     self.restart_ks()
         elif action == "notify_power_changed":
@@ -800,31 +858,32 @@ class KlipperScreen(Gtk.Window):
             self.printer.process_power_update(data)
             self.panels['splash_screen'].check_power_status()
         elif action == "notify_gcode_response" and self.printer.state not in ["error", "shutdown"]:
-            if not (data.startswith("B:") or data.startswith("T:")):
-                if data.startswith("// action:"):
-                    action = data[10:]
-                    if action.startswith('prompt_begin'):
-                        if self.prompt is not None:
-                            self.prompt.end()
-                        self.prompt = Prompt(self)
-                    if self.prompt is None:
-                        return
-                    self.prompt.decode(action)
-                elif data.startswith("echo: "):
-                    self.show_popup_message(data[6:], 1)
-                elif data.startswith("!! "):
-                    self.show_popup_message(data[3:], 3)
-                elif "unknown" in data.lower() and \
-                        not ("TESTZ" in data or "MEASURE_AXES_NOISE" in data or "ACCELEROMETER_QUERY" in data):
-                    self.show_popup_message(data)
-                elif "SAVE_CONFIG" in data and self.printer.state == "ready":
-                    script = {"script": "SAVE_CONFIG"}
-                    self._confirm_send_action(
-                        None,
-                        _("Save configuration?") + "\n\n" + _("Klipper will reboot"),
-                        "printer.gcode.script",
-                        script
-                    )
+            if re.match('^(?:ok\\s+)?(B|C|T\\d*):', data):
+                return
+            if data.startswith("// action:"):
+                action = data[10:]
+                if action.startswith('prompt_begin'):
+                    if self.prompt is not None:
+                        self.prompt.end()
+                    self.prompt = Prompt(self)
+                if self.prompt is None:
+                    return
+                self.prompt.decode(action)
+            elif data.startswith("echo: "):
+                self.show_popup_message(data[6:], 1, from_ws=True)
+            elif data.startswith("!! "):
+                self.show_popup_message(data[3:], 3, from_ws=True)
+            elif "unknown" in data.lower() and \
+                    not ("TESTZ" in data or "MEASURE_AXES_NOISE" in data or "ACCELEROMETER_QUERY" in data):
+                self.show_popup_message(data, from_ws=True)
+            elif "SAVE_CONFIG" in data and self.printer.state == "ready":
+                script = {"script": "SAVE_CONFIG"}
+                self._confirm_send_action(
+                    None,
+                    _("Save configuration?") + "\n\n" + _("Klipper will reboot"),
+                    "printer.gcode.script",
+                    script
+                )
         self.process_update(action, data)
 
     def process_update(self, *args):
@@ -874,7 +933,7 @@ class KlipperScreen(Gtk.Window):
 
     def printer_initializing(self, msg, go_to_splash=False):
         if 'splash_screen' not in self.panels or go_to_splash:
-            self.show_panel("splash_screen", None, remove_all=True)
+            self.show_panel("splash_screen", remove_all=True)
         self.panels['splash_screen'].update_text(msg)
         self.log_notification(msg, 0)
 
@@ -1138,11 +1197,6 @@ class KlipperScreen(Gtk.Window):
 
 
 def main():
-    minimum = (3, 7)
-    if not sys.version_info >= minimum:
-        logging.error(f"python {sys.version_info.major}.{sys.version_info.minor} "
-                      f"does not meet the minimum requirement {minimum[0]}.{minimum[1]}")
-        sys.exit(1)
     parser = argparse.ArgumentParser(description="KlipperScreen - A GUI for Klipper")
     homedir = os.path.expanduser("~")
 
@@ -1184,4 +1238,4 @@ if __name__ == "__main__":
         main()
     except Exception as ex:
         logging.exception(f"Fatal error in main loop:\n{ex}\n\n{traceback.format_exc()}")
-        sys.exit(1)
+        os._exit(1)
